#property copyright "Copyright  2005, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net/"

#property indicator_chart_window
#property indicator_buffers 7
#property indicator_color1 Yellow
#property indicator_color2 Lime
#property indicator_color3 Lime
#property indicator_color4 Lime
#property indicator_color5 Lime
#property indicator_color6 Lime
#property indicator_color7 Lime
#property indicator_width1 3

//---- indicator parameters
extern string TimeFrame         = "Current time frame";
extern int    BandsPeriod       = 20;
extern int    BandsShift        = 0;
extern double BandsDeviations   = 1.0;
extern double BandsDeviations1  = 2.0;
extern double BandsDeviations2  = 3.0;
//---- buffers
double MovingBuffer[];
double UpperBuffer[];
double LowerBuffer[];
double UpperBuffer1[];
double LowerBuffer1[];
double UpperBuffer2[];
double LowerBuffer2[];
string IndicatorFileName;
int    timeFrame;
bool   returnBars;
bool   calculateValue;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   SetIndexBuffer(0,MovingBuffer);
   SetIndexBuffer(1,UpperBuffer);
   SetIndexBuffer(2,LowerBuffer);
   SetIndexBuffer(3,UpperBuffer1);
   SetIndexBuffer(4,LowerBuffer1);
   SetIndexBuffer(5,UpperBuffer2);
   SetIndexBuffer(6,LowerBuffer2);
   
      IndicatorFileName = WindowExpertName();
      returnBars        = (TimeFrame == "returnBars");   if (returnBars)     return(0);
      calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);   
   
   SetIndexDrawBegin(0,BandsPeriod+BandsShift);
   SetIndexDrawBegin(1,BandsPeriod+BandsShift);
   SetIndexDrawBegin(2,BandsPeriod+BandsShift);
   SetIndexDrawBegin(3,BandsPeriod+BandsShift);
   SetIndexDrawBegin(4,BandsPeriod+BandsShift);
   SetIndexDrawBegin(5,BandsPeriod+BandsShift);
   SetIndexDrawBegin(6,BandsPeriod+BandsShift);
   return(0);
}
//+------------------------------------------------------------------+
//| Bollinger Bands                                                  |
//+------------------------------------------------------------------+
int start()
{
   int counted_bars = IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
          int limit=MathMin(Bars-counted_bars,Bars-1);
          if (returnBars) { MovingBuffer[0] = limit; return(0); }

   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame == Period())
   {
      for(int i=limit; i>=0; i--)
      {
         MovingBuffer[i]=iMA(NULL,0,BandsPeriod,BandsShift,MODE_EMA,PRICE_CLOSE,i);
         double sum=0.0;
            for(int k=0; k<BandsPeriod; k++) sum+=(Close[i+k]-MovingBuffer[i])*(Close[i+k]-MovingBuffer[i]);
            double deviation  = BandsDeviations *MathSqrt(sum/BandsPeriod);
            double deviation1 = BandsDeviations1*MathSqrt(sum/BandsPeriod);
            double deviation2 = BandsDeviations2*MathSqrt(sum/BandsPeriod);
      
            UpperBuffer[i]  = MovingBuffer[i]+deviation;
            LowerBuffer[i]  = MovingBuffer[i]-deviation;
            UpperBuffer1[i] = MovingBuffer[i]+deviation1;
            LowerBuffer1[i] = MovingBuffer[i]-deviation1;
            UpperBuffer2[i] = MovingBuffer[i]+deviation2;
            LowerBuffer2[i] = MovingBuffer[i]-deviation2;
      }
      return(0);
   }
   
   //
   //
   //
   //
   //

   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         MovingBuffer[i] = iCustom(NULL,timeFrame,IndicatorFileName,"calculateValue",BandsPeriod,BandsShift,BandsDeviations,BandsDeviations1,BandsDeviations2,0,y);
         UpperBuffer[i]  = iCustom(NULL,timeFrame,IndicatorFileName,"calculateValue",BandsPeriod,BandsShift,BandsDeviations,BandsDeviations1,BandsDeviations2,1,y);
         LowerBuffer[i]  = iCustom(NULL,timeFrame,IndicatorFileName,"calculateValue",BandsPeriod,BandsShift,BandsDeviations,BandsDeviations1,BandsDeviations2,2,y);
         UpperBuffer1[i] = iCustom(NULL,timeFrame,IndicatorFileName,"calculateValue",BandsPeriod,BandsShift,BandsDeviations,BandsDeviations1,BandsDeviations2,3,y);
         LowerBuffer1[i] = iCustom(NULL,timeFrame,IndicatorFileName,"calculateValue",BandsPeriod,BandsShift,BandsDeviations,BandsDeviations1,BandsDeviations2,4,y);
         UpperBuffer2[i] = iCustom(NULL,timeFrame,IndicatorFileName,"calculateValue",BandsPeriod,BandsShift,BandsDeviations,BandsDeviations1,BandsDeviations2,5,y);
         LowerBuffer2[i] = iCustom(NULL,timeFrame,IndicatorFileName,"calculateValue",BandsPeriod,BandsShift,BandsDeviations,BandsDeviations1,BandsDeviations2,6,y);
   }         
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

